package com.hys.app.service.erp.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.hys.app.converter.erp.GoodsConverter;
import com.hys.app.framework.database.WebPage;
import com.hys.app.framework.database.mybatisplus.base.BaseServiceImpl;
import com.hys.app.framework.exception.ServiceException;
import com.hys.app.framework.util.BeanUtils;
import com.hys.app.mapper.goods.GoodsMapper;
import com.hys.app.model.erp.dos.GoodsDO;
import com.hys.app.model.erp.dos.ProductDO;
import com.hys.app.model.erp.dos.ProductStockDO;
import com.hys.app.model.erp.dto.GoodsDTO;
import com.hys.app.model.erp.dto.GoodsQueryParams;
import com.hys.app.model.erp.dto.ProductDTO;
import com.hys.app.model.erp.vo.GoodsVO;
import com.hys.app.model.erp.vo.ProductStockVO;
import com.hys.app.model.erp.vo.ProductVO;
import com.hys.app.model.goods.dos.GoodsGalleryDO;
import com.hys.app.service.erp.GoodsManager;
import com.hys.app.service.erp.ProductManager;
import com.hys.app.service.erp.ProductStockManager;
import com.hys.app.service.goods.GoodsGalleryManager;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 商品业务层实现
 *
 * @author 张崧
 * 2023-12-26 15:56:58
 */
@Service
@AllArgsConstructor
public class GoodsManagerImpl extends BaseServiceImpl<GoodsMapper, GoodsDO> implements GoodsManager {

    private GoodsConverter converter;
    private ProductStockManager productStockManager;
    private ProductManager productManager;
    private GoodsGalleryManager goodsGalleryManager;

    @Override
    public WebPage<GoodsVO> list(GoodsQueryParams queryParams) {
        WebPage<GoodsDO> webPage = baseMapper.selectPage(queryParams);
        // 查询商品标签
        return converter.convert(webPage);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void add(GoodsDTO goodsDTO) {
        checkParams(goodsDTO);
        GoodsDO goodsDO = BeanUtil.copyProperties(goodsDTO, GoodsDO.class);
        List<ProductDO> bean = BeanUtils.toBean(goodsDTO.getProductList(), ProductDO.class);
        super.save(goodsDO);
        bean.forEach(item -> item.setGoodsId(goodsDO.getId()));
        productManager.saveBatch(bean);
        // 保存相册
        this.goodsGalleryManager.add(goodsDTO.getImageList(), goodsDO.getId());
        //初始化库存
        productStockManager.intStock(bean);
        ;

        // 发送商品消息
//        GoodsChangeMessage message = new GoodsChangeMessage(GoodsChangeMessage.GoodsChangeType.Add, goodsId);
//        this.messageSender.send(new MqMessage(AmqpExchange.ERP_GOODS_CHANGE, AmqpExchange.ERP_GOODS_CHANGE + "_ROUTING", message));
    }




    @Override
    @Transactional(rollbackFor = Exception.class)
    public void edit(GoodsDTO goodsDTO) {
        checkParams(goodsDTO);
        // 更新商品
        GoodsDO aDo = BeanUtil.copyProperties(goodsDTO, GoodsDO.class);
        //先删除sku在新增
        productManager.remove(new LambdaQueryWrapper<ProductDO>().eq(ProductDO::getGoodsId, goodsDTO.getId()));
        List<ProductDO> bean = BeanUtils.toBean(goodsDTO.getProductList(), ProductDO.class);
        updateById(aDo);
        bean.forEach(item -> item.setGoodsId(goodsDTO.getId()));
        productManager.saveBatch(bean);
        goodsGalleryManager.edit(goodsDTO.getImageList(), goodsDTO.getId());

        // 发送商品消息
//        GoodsChangeMessage message = new GoodsChangeMessage(GoodsChangeMessage.GoodsChangeType.Edit, goodsDTO.getId());
//        this.messageSender.send(new MqMessage(AmqpExchange.ERP_GOODS_CHANGE, AmqpExchange.ERP_GOODS_CHANGE + "_ROUTING", message));
    }


    /**
     * 校验参数   sku不能重复   条形码不能重复   商品名称不能重复
     *
     * @param goodsDTO
     */
    private void checkParams(GoodsDTO goodsDTO) {
        GoodsDO aDo = getOne(new LambdaQueryWrapper<GoodsDO>()
                .eq(GoodsDO::getName, goodsDTO.getName())
                .ne(ObjectUtil.isNotNull(goodsDTO.getId()), GoodsDO::getId, goodsDTO.getId())
                .last("limit 1"));
        if (ObjectUtil.isNotNull(aDo)) {
            throw new ServiceException("商品名称已存在");
        }
        for (ProductDTO productDTO : goodsDTO.getProductList()) {
            ProductDO productDO = productManager.getOne(new LambdaQueryWrapper<ProductDO>()
                    .eq(ProductDO::getSku, productDTO.getSku())
                    .ne(ObjectUtil.isNotNull(productDTO.getId()), ProductDO::getId, productDTO.getId())
                    .last("limit 1"));
            if (ObjectUtil.isNotNull(productDO)) {
                throw new ServiceException("sku已存在");
            }

            ProductDO codeOne = productManager.getOne(new LambdaQueryWrapper<ProductDO>()
                    .eq(ProductDO::getBarCode, productDTO.getBarCode())
                    .ne(ObjectUtil.isNotNull(productDTO.getId()), ProductDO::getId, productDTO.getId())
                    .last("limit 1"));
            if (ObjectUtil.isNotNull(codeOne)) {
                throw new ServiceException("条码已存在");
            }
        }
    }



    @Override
    public GoodsVO getInfo(Long id) {
        GoodsDO goodsDO = getById(id);
        List<ProductDO> list = productManager.list(new LambdaQueryWrapper<ProductDO>().eq(ProductDO::getGoodsId, id));
        List<ProductDTO> bean = BeanUtils.toBean(list, ProductDTO.class);
        GoodsVO goodsVO = BeanUtil.copyProperties(goodsDO, GoodsVO.class);
        goodsVO.setProductList(bean);
        return goodsVO;
    }

    @Override
    public GoodsVO getDetail(Long id) {
        // 商品
        GoodsDO goodsDO = getById(id);
        List<ProductDO> list = productManager.list(new LambdaQueryWrapper<ProductDO>().eq(ProductDO::getGoodsId, id));
        List<ProductDTO> bean = BeanUtils.toBean(list, ProductDTO.class);
        GoodsVO goodsVO = BeanUtil.copyProperties(goodsDO, GoodsVO.class);
        goodsVO.setProductList(bean);
        // 商品相册
        List<GoodsGalleryDO> galleryList = goodsGalleryManager.list(goodsDO.getId());
        goodsVO.setImageList(galleryList);
        return goodsVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delete(Long id) {
        GoodsDO goodsDO = getById(id);
        if (goodsDO.getMarketEnable()) {
            throw new ServiceException("请先下架商品");
        }
        removeById(id);
    }

    @Override
    public long countByCategoryId(Long categoryId) {
        return this.baseMapper.selectCount(new LambdaQueryWrapper<GoodsDO>().eq(GoodsDO::getCategoryId, categoryId));
    }

    @Override
    public Boolean updateCheck(Long id) {
        ProductStockDO one = productStockManager.getOne(new LambdaQueryWrapper<ProductStockDO>().eq(ProductStockDO::getGoodsId, id).gt(ProductStockDO::getUsableStock, 0));
        if (ObjectUtil.isNotNull(one)) {
            return true;
        }

        return false;
    }

    @Override
    public List<ProductVO> getSkuByWarehouse() {
        List<ProductDO> productDOS = productManager.list();
        List<ProductVO> voList = BeanUtils.toBean(productDOS, ProductVO.class);
        Set<Long> set = productDOS.stream().map(ProductDO::getGoodsId).collect(Collectors.toSet());
        List<GoodsDO> goodsDOS = this.baseMapper.selectList(new LambdaQueryWrapper<GoodsDO>().in(GoodsDO::getId, set));
        for (ProductVO vo : voList) {
            GoodsDO goodsDO = goodsDOS.stream().filter(item -> item.getId().equals(vo.getGoodsId())).findFirst().get();
            GoodsVO goodsVO = BeanUtil.copyProperties(goodsDO, GoodsVO.class);
            vo.setGoodsVo(goodsVO);
        }
        return voList;
    }
}

